package com.allpay.charge.exception.handler;

import cn.hutool.core.util.StrUtil;
import com.allpay.charge.exception.IllegalOperationException;
import com.allpay.charge.exception.ProcessingFailureException;
import com.allpay.charge.exception.UnauthenticatedException;
import com.allpay.charge.tools.ThrowableUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.stream.Collectors;

import static org.springframework.http.HttpStatus.*;

/**
 * @Description :
 * @Author : Simon.shi
 * @Date : 2019/8/13 14:25
 * @Version : V0.0.1
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

    /**
     * 处理所有不可知的异常
     * @param e
     * @return
     */
    @ExceptionHandler(Throwable.class)
    public ResponseEntity<ApiError> handleException(Throwable e){
        // 打印堆栈信息
        log.error(ThrowableUtil.getStackTrace(e));
        ApiError apiError = new ApiError(BAD_REQUEST.value(), StrUtil.isBlank(e.getMessage()) ?"系统异常":e.getMessage());
        return buildResponseT(apiError);
    }

    /**
     * 处理非法的操作异常
     * @param e
     * @return
     */
    @ExceptionHandler(UnauthenticatedException.class)
    public ResponseEntity<ApiError> handleException(UnauthenticatedException e){
        log.error(ThrowableUtil.getStackTrace(e));
        ApiError apiError = new ApiError(FORBIDDEN.value(),e.getMessage());
        return buildResponseT(apiError);
    }

    /**
     * 处理非法的操作异常
     * @param e
     * @return
     */
    @ExceptionHandler(IllegalOperationException.class)
    public ResponseEntity<ApiError> handleException(IllegalOperationException e){
        log.error(ThrowableUtil.getStackTrace(e));
        ApiError apiError = new ApiError(METHOD_NOT_ALLOWED.value(),e.getMessage());
        return buildResponseT(apiError);
    }

    //处理Get请求中 使用@Valid 验证路径中请求实体校验失败后抛出的异常，详情继续往下看代码
    @ExceptionHandler(BindException.class)
    public ResponseEntity<ApiError> BindExceptionHandler(BindException e) {
        log.error(ThrowableUtil.getStackTrace(e));
        String message = e.getBindingResult().getAllErrors().stream().map(DefaultMessageSourceResolvable::getDefaultMessage).collect(Collectors.joining());
        ApiError apiError = new ApiError(BAD_REQUEST.value(),message);
        return buildResponseT(apiError);
    }

    //处理请求参数格式错误 @RequestParam上validate失败后抛出的异常是javax.validation.ConstraintViolationException
    @ExceptionHandler(ConstraintViolationException.class)
    public ResponseEntity<ApiError> ConstraintViolationExceptionHandler(ConstraintViolationException e) {
        String message = e.getConstraintViolations().stream().map(ConstraintViolation::getMessage).collect(Collectors.joining());
        log.error(ThrowableUtil.getStackTrace(e));
        ApiError apiError = new ApiError(BAD_REQUEST.value(),message);
        return buildResponseT(apiError);
    }

    //处理请求参数格式错误 @RequestBody上validate失败后抛出的异常是MethodArgumentNotValidException异常。
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseEntity<ApiError> MethodArgumentNotValidExceptionHandler(MethodArgumentNotValidException e) {
        String message = e.getBindingResult().getAllErrors().stream().map(DefaultMessageSourceResolvable::getDefaultMessage).collect(Collectors.joining());
        log.error(ThrowableUtil.getStackTrace(e));
        ApiError apiError = new ApiError(BAD_REQUEST.value(),message);
        return buildResponseT(apiError);
    }

    //处理失败异常
    @ExceptionHandler(ProcessingFailureException.class)
    public ResponseEntity<ApiSuccess> ProcessingFailureExceptionHandler(ProcessingFailureException e) {
        log.error(ThrowableUtil.getStackTrace(e));
        ApiSuccess apiSuccess = new ApiSuccess<>(ApiSuccess.FAIL_CODE,e.getMessage());
        return ResponseEntity.ok(apiSuccess);
    }

    private ResponseEntity<ApiError> buildResponseT(ApiError apiError) {
        return new ResponseEntity<ApiError>(apiError, HttpStatus.valueOf(apiError.getStatus()));
    }
}
